package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.huifu.adapay.core.exception.BaseAdaPayException;
import com.microsvc.toolkit.middleware.payment.common.api.BasePaymentService;
import com.microsvc.toolkit.middleware.payment.common.api.PaymentServiceContext;
import com.microsvc.toolkit.middleware.payment.common.api.entity.PaymentMerchant;
import com.ym.mec.biz.dal.config.AdapayNotifyConstants;
import com.ym.mec.biz.dal.dao.HfMemberDao;
import com.ym.mec.biz.dal.dto.YeepayMember;
import com.ym.mec.biz.dal.entity.HfMember;
import com.ym.mec.biz.dal.enums.PaymentChannelEnum;
import com.ym.mec.biz.service.HfMemberService;
import com.ym.mec.biz.service.HfMerchantConfigService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.thirdparty.adapay.CorpMemberService;
import com.ym.mec.thirdparty.adapay.SettleAccountService;
import com.ym.mec.thirdparty.adapay.entity.AccountInfo;
import com.ym.mec.thirdparty.adapay.entity.EnterpriseUser;
import com.ym.mec.thirdparty.adapay.entity.HfMerchantConfig;
import com.ym.mec.thirdparty.exception.ThirdpartyException;
import com.ym.mec.util.string.MessageFormatter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Service
@Slf4j
public class HfMemberServiceImpl extends BaseServiceImpl<Integer, HfMember> implements HfMemberService {
	
    @Resource
    private HfMemberDao hfMemberDao;
    @Resource
    private HfMerchantConfigService hfMerchantConfigService;
    @Resource
    private PaymentServiceContext paymentServiceContext;

    @Value("${spring.profiles.active:dev}")
    private String env;


    @Override
    public BaseDAO<Integer, HfMember> getDAO() {
        return hfMemberDao;
    }

    @Override
	public HfMember getByMemberId(String memberId, String payerName) {
		return hfMemberDao.getByMemberId(memberId, payerName);
	}

	@Override
	public HfMember getByOrganId(Integer organId, String payerName) {
		return hfMemberDao.getByOrganId(organId, payerName);
	}

	@Override
	public HfMember queryByRequestNo(String requestNo) {
		return hfMemberDao.queryByRequestNo(requestNo);
	}

	@Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createMember(HfMember member, File file) throws Exception {
        HfMember hasMember = hfMemberDao.getByOrganId(member.getOrganId(), member.getPayerName());
        if (hasMember != null) {
            throw new BizException("该分部已创建成功，请核查");
        }
        hasMember = hfMemberDao.getByName(member.getName(), member.getPayerName());
        if (hasMember != null) {
            throw new BizException("公司名与已有公司名相同，请核查");
        }
        BasePaymentService paymentService = paymentServiceContext.getPaymentService(member.getPayerName() + "_" + member.getTenantId());
        if (paymentService == null) {
            throw new BizException("支付渠道不存在:{}", member.getPayerName());
        }
        PaymentMerchant.MerchantConfig merchantConfig = getMerchantConfig(member.getPayerName() + "_" + member.getTenantId(),member.getTenantId());
        PaymentMerchant.MerchantMember merchantMember = getHfMerchantMember(member);
        merchantMember.setAttachFile(file);
        try {
            log.info("创建子账户参数:{}", JSON.toJSONString(merchantMember));
            paymentService.createPaymentCorpMember(merchantConfig, merchantMember);
        }catch (Exception e) {
            log.error("创建子账户失败", e);
            throw new BizException("创建子账户失败:" + e.getMessage());
        }
/*
        switch (PaymentChannelEnum.codeOf(member.getPayerName())) {
		case ADAPAY:
			createMemberWithAdapay(member, file);
			break;
			
		case YEEPAY:
			Map<String, Object> response = createMemberWithYeepay((YeepayMember) member);
			member.setRequestNo(response.get("requestNo").toString());
			member.setMemberId(response.get("merchantNo").toString());
			break;

		default:
			
			return false;
		}*/

        Date nowDate = new Date();
        member.setCreateTime(nowDate);
        member.setUpdateTime(nowDate);
        member.setStatus("pending");
        insert(member);

        return true;
    }

    @Override
    public PaymentMerchant.MerchantConfig getMerchantConfig(String paymentVerdor, Integer tenantId) {
        HfMerchantConfig merchantConfig = hfMerchantConfigService.queryByTenantId(tenantId,paymentVerdor);
        if (merchantConfig == null) {
            throw new BizException("机构[{}]商户信息找不到", tenantId);
        }
        return PaymentMerchant.MerchantConfig.builder()
                .payerName(paymentVerdor)
                .appId(merchantConfig.getAppId())
                .merchantKey(merchantConfig.getMerKey())
                .apiKey(merchantConfig.getApiKey())
                .rsaPrivateKey(merchantConfig.getRsaPrivateKey())
                .rsaPublicKey(merchantConfig.getRsaPublicKey())
                .wxAppId(merchantConfig.getWxAppId())
                .wxAppSecret(merchantConfig.getWxAppSecret())
                .alipayAppId(merchantConfig.getAlipayAppId())
                .alipayPublicKey(merchantConfig.getAlipayPublicKey())
                .platformPayeeMemberId(merchantConfig.getPlatformPayeeMemberId())
                .build();
    }

    private PaymentMerchant.MerchantMember getHfMerchantMember(HfMember tenantMember) {
        if (StringUtils.equalsIgnoreCase(tenantMember.getPayerName(), PaymentChannelEnum.ADAPAY.getCode())) {
            PaymentMerchant.MerchantMember merchantMember = PaymentMerchant.MerchantMember.builder()
                    .memberId(tenantMember.getMemberId())
                    .merchantName(tenantMember.getName())
                    .provinceCode(tenantMember.getProvCode())
                    .cityCode(tenantMember.getAreaCode())
                    .districtCode(tenantMember.getDistrictCode())
                    .socialCreditCode(tenantMember.getSocialCreditCode())
                    .socialCreditCodeExpires(tenantMember.getSocialCreditCodeExpires())
                    .businessScope(tenantMember.getBusinessScope())
                    .legalName(tenantMember.getLegalPerson())
                    .legalLicenceNo(tenantMember.getLegalCertId())
                    .legalCertIdExpires(tenantMember.getLegalCertIdExpires())
                    .mobile(tenantMember.getLegalMp())
                    .legalLicenceType(tenantMember.getLegalLicenceType())
                    .address(tenantMember.getAddress())
                    .bankCode(tenantMember.getBankCode())
                    .bankAccountType(tenantMember.getBankAcctType())
                    .bankCardNo(tenantMember.getCardNo())
                    .cardName(tenantMember.getCardName())
                    .zipCode(tenantMember.getZipCode())
                    .requestNo(tenantMember.getRequestNo())
                    .licenceNo(tenantMember.getSocialCreditCode())
                    .contactName(tenantMember.getContactName())
                    .contactMobile(tenantMember.getContactMobile())
                    .contactEmail(tenantMember.getContactEmail())
                    .contactLicenceNo(tenantMember.getContactLicenceNo())
                    .primaryIndustryCategory("120")
                    .secondaryIndustryCategory("120004")
                    .settlementDirection("BANKCARD")
                    .build();
            if (StringUtils.equals(env, "prod") || StringUtils.equals(env, "pre")) {
                merchantMember.setNotifyUrl(MessageFormatter.arrayFormat(AdapayNotifyConstants.corp_member_notify_url_prod, "adapay"));
            } else {
                merchantMember.setNotifyUrl(MessageFormatter.arrayFormat(AdapayNotifyConstants.corp_member_notify_url_test, "adapay"));
            }
            return merchantMember;
        } else {
            YeepayMember yeepayMember = (YeepayMember) tenantMember;
            PaymentMerchant.MerchantMember merchantMember = PaymentMerchant.MerchantMember.builder()
                    .memberId(yeepayMember.getMemberId())
                    .merchantName(yeepayMember.getName())
                    .merchantType(yeepayMember.getMerchantType())
                    .provinceCode(yeepayMember.getProvCode())
                    .cityCode(yeepayMember.getAreaCode())
                    .districtCode(yeepayMember.getDistrictCode())
                    .socialCreditCode(yeepayMember.getSocialCreditCode())
                    .socialCreditCodeExpires(yeepayMember.getSocialCreditCodeExpires())
                    .businessScope(yeepayMember.getBusinessScope())
                    .legalName(yeepayMember.getLegalPerson())
                    .legalLicenceNo(yeepayMember.getLegalCertId())
                    .legalCertIdExpires(yeepayMember.getLegalCertIdExpires())
                    .mobile(yeepayMember.getLegalMp())
                    .legalLicenceType(yeepayMember.getLegalLicenceType())
                    .legalLicenceFrontUrl(yeepayMember.getLegalLicenceFrontUrl())
                    .legalLicenceBackUrl(yeepayMember.getLegalLicenceBackUrl())
                    .address(yeepayMember.getAddress())
                    .bankCode(yeepayMember.getBankCode())
                    .bankAccountType(yeepayMember.getBankAcctType())
                    .bankCardNo(yeepayMember.getCardNo())
                    .cardName(yeepayMember.getCardName())
                    .zipCode(yeepayMember.getZipCode())
                    .requestNo(yeepayMember.getRequestNo())
                    .licenceNo(yeepayMember.getSocialCreditCode())
                    .licenceUrl(yeepayMember.getLicenceUrl())
                    .openAccountLicenceUrl(yeepayMember.getOpenAccountLicenceUrl())
                    .contactName(yeepayMember.getContactName())
                    .contactMobile(yeepayMember.getContactMobile())
                    .contactEmail(yeepayMember.getContactEmail())
                    .contactLicenceNo(yeepayMember.getContactLicenceNo())
                    .primaryIndustryCategory("120")
                    .secondaryIndustryCategory("120004")
                    .settlementDirection("BANKCARD")
                    .build();
            String notifyURL = MessageFormatter.arrayFormat(AdapayNotifyConstants.corp_member_notify_url_test, "yeepay");
            if (StringUtils.equals(env, "prod") || StringUtils.equals(env, "pre")) {
                notifyURL = MessageFormatter.arrayFormat(AdapayNotifyConstants.corp_member_notify_url_prod, "yeepay");
            }
            merchantMember.setNotifyUrl(notifyURL);
            return merchantMember;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateMember(HfMember member, File file) throws Exception {
        HfMember hfMember = hfMemberDao.get(member.getId());
        if (hfMember == null) {
            throw new BizException("商户不存在请核对");
        }
        BasePaymentService paymentService = paymentServiceContext.getPaymentService(member.getPayerName() + "_" + member.getTenantId());
        if (paymentService == null) {
            throw new BizException("支付渠道不存在:{}", member.getPayerName());
        }
        PaymentMerchant.MerchantConfig merchantConfig = getMerchantConfig(member.getPayerName() + "_" + member.getTenantId(),member.getTenantId());
        PaymentMerchant.MerchantMember merchantMember = getHfMerchantMember(member);
        merchantMember.setAttachFile(file);
        try {
            log.info("更新商户信息失败:{}", JSON.toJSONString(merchantMember));
            paymentService.updatePaymentCorpMember(merchantConfig, merchantMember);
        }catch (Exception e) {
            log.error("更新商户信息失败", e);
            throw new BizException("更新商户信息失败:" + e.getMessage());
        }
        /*Integer tenantId = member.getTenantId();

        
        HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(tenantId, member.getPayerName());
        if(hfMerchantConfig == null){
        	throw new BizException("机构[{}]汇付商户信息找不到", tenantId);
        }
        
        EnterpriseUser enterpriseUser = new EnterpriseUser();
        enterpriseUser.setAddress(member.getAddress());
        enterpriseUser.setAreaCode(member.getAreaCode());
        enterpriseUser.setAttachFile(file);
        enterpriseUser.setBankAcctType(member.getBankAcctType());
        enterpriseUser.setBankCode(member.getBankCode());
        enterpriseUser.setBusinessScope(member.getBusinessScope());
        enterpriseUser.setCardName(member.getCardName());
        enterpriseUser.setCardNo(member.getCardNo());
        enterpriseUser.setLegalCertId(member.getLegalCertId());
        enterpriseUser.setLegalCertIdExpires(member.getLegalCertIdExpires());
        enterpriseUser.setLegalMp(member.getLegalMp());
        enterpriseUser.setLegalPerson(member.getLegalPerson());
        enterpriseUser.setMemberId(member.getMemberId());
        enterpriseUser.setName(member.getName());
        
        if(StringUtils.equals(env, "prod") || StringUtils.equals(env, "pre")){
            enterpriseUser.setNotifyUrl(AdapayNotifyConstants.corp_member_notify_url_prod);
        }else {
            enterpriseUser.setNotifyUrl(AdapayNotifyConstants.corp_member_notify_url_test);
        }
        
        enterpriseUser.setProvCode(member.getProvCode());
        enterpriseUser.setSocialCreditCode(member.getSocialCreditCode());
        enterpriseUser.setSocialCreditCodeExpires(member.getSocialCreditCodeExpires());
        
        corpMemberService.update(hfMerchantConfig.getMerKey(), hfMerchantConfig.getAppId(), enterpriseUser, enterpriseUser.getNotifyUrl());
*/
        Date nowDate = new Date();
        member.setUpdateTime(nowDate);
        member.setStatus("pending");
        if (update(member) <= 0) {
            throw new BizException("更新商户信息失败,请重试");
        }
        
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean mergeSettleAccount(String memberId, AccountInfo accountInfo) throws Exception {
        HfMember hfMember = hfMemberDao.getByMemberId(memberId, PaymentChannelEnum.ADAPAY.getCode());
        if (!hfMember.getStatus().equals("succeeded")) {
            throw new BizException("商户审核通过，才能绑定结算卡");
        }
        accountInfo.setAreaCode(hfMember.getAreaCode());
        accountInfo.setBankAcctType(hfMember.getBankAcctType());
        accountInfo.setBankCode(hfMember.getBankCode());
        accountInfo.setBankName(hfMember.getBankCode());
        accountInfo.setCardId(hfMember.getCardNo());
        accountInfo.setCardName(hfMember.getCardName());
        accountInfo.setMemberId(memberId);
        accountInfo.setProvCode(hfMember.getProvCode());
        accountInfo.setTelNo(hfMember.getLegalMp());
        
        Integer tenantId = hfMember.getTenantId();
        
        HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(tenantId, PaymentChannelEnum.ADAPAY.getCode());
        if(hfMerchantConfig == null){
        	throw new BizException("机构[{}]汇付商户信息找不到", tenantId);
        }
        BasePaymentService paymentService = paymentServiceContext.getPaymentService(PaymentChannelEnum.ADAPAY.getCode() + "_" + TenantContextHolder.getTenantId());
        if (paymentService == null) {
            throw new BizException("支付渠道不存在:{}", PaymentChannelEnum.ADAPAY.getCode() + "_" + TenantContextHolder.getTenantId());
        }

        //删除结算账户
        if (StringUtils.isNotBlank(hfMember.getSettleAccountId())) {
//            settleAccountService.delete(hfMerchantConfig.getMerKey(), hfMerchantConfig.getAppId(), memberId, hfMember.getSettleAccountId());
            paymentService.deleteSettleAccount(hfMerchantConfig.getMerKey(), hfMerchantConfig.getAppId(), memberId, hfMember.getSettleAccountId());
        }

        //创建结算账户
//        String settleAccountId = settleAccountService.create(hfMerchantConfig.getMerKey(), hfMerchantConfig.getAppId(), accountInfo);
        PaymentMerchant.MerchantConfig merchantConfig = getMerchantConfig(PaymentChannelEnum.ADAPAY.getCode(), TenantContextHolder.getTenantId());
        String settleAccount = paymentService.createSettleAccount(merchantConfig, getAccountInfo(accountInfo));

        hfMember.setSettleAccountId(settleAccount);
        if (hfMemberDao.update(hfMember) <= 0) {
            throw new BizException("绑卡失败，请重试");
        }
        return true;
    }

    private PaymentMerchant.AccountInfo getAccountInfo(AccountInfo accountInfo) {
        return PaymentMerchant.AccountInfo.builder()
                .memberId(accountInfo.getMemberId())
                .cardId(accountInfo.getCardId())
                .cardName(accountInfo.getCardName())
                .certId(accountInfo.getCertId())
                .certType(accountInfo.getCertType())
                .telNo(accountInfo.getTelNo())
                .bankCode(accountInfo.getBankCode())
                .bankName(accountInfo.getBankName())
                .bankAcctType(accountInfo.getBankAcctType())
                .provCode(accountInfo.getProvCode())
                .areaCode(accountInfo.getAreaCode())
                .build();
    }

    @Override
    public Map<String, Object> queryWithKey(String appId, String merKey, int pageIndex, long createdGte, long createdLte) {
        Map<java.lang.String, java.lang.Object> paymentParams = new HashMap<>();
        paymentParams.put("app_id", appId);
        paymentParams.put("page_index", pageIndex);
        paymentParams.put("page_size", "20");
        paymentParams.put("created_gte", createdGte);
        paymentParams.put("created_lte", createdLte);
        Map<String, Object> payment = null;
        try {
            payment = com.huifu.adapay.model.Payment.queryList(paymentParams, merKey);
        } catch (BaseAdaPayException e) {
            throw new BizException("查询支付对象列表失败:{}", e, e.getMessage());
        }
        String error_code = (String) payment.get("error_code");
        if (null != error_code) {
            String errorMsg = (String) payment.get("error_msg");
            throw new ThirdpartyException(errorMsg);
        }
        return payment;
    }

    @Override
    public Map<String, Object> queryConfirmList(String appId, String merchantKey, String paymentId){
        Map<String, Object> paymentParams = new HashMap<>();
        paymentParams.put("app_id", appId);
        paymentParams.put("payment_id", paymentId);

        Map<String, Object> payment = null;
        try {
            payment = com.huifu.adapay.model.Payment.queryConfirmList(paymentParams, merchantKey);
        } catch (BaseAdaPayException e) {
            throw new BizException("查询支付确认列表失败:{}", e, e.getMessage());
        }
        String error_code = (String) payment.get("error_code");
        if (null != error_code) {
            String errorMsg = (String) payment.get("error_msg");
            throw new ThirdpartyException(errorMsg);
        }
        return payment;
    }
    /*
	private boolean createMemberWithAdapay(HfMember member, File file) {
		Integer tenantId = member.getTenantId();

		HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(tenantId, member.getPayerName());
		if (hfMerchantConfig == null) {
			throw new BizException("机构[{}]汇付商户信息找不到", tenantId);
		}

		EnterpriseUser enterpriseUser = new EnterpriseUser();
		enterpriseUser.setAddress(member.getAddress());
		enterpriseUser.setAreaCode(member.getAreaCode());
		enterpriseUser.setAttachFile(file);
		enterpriseUser.setBankAcctType(member.getBankAcctType());
		enterpriseUser.setBankCode(member.getBankCode());
		enterpriseUser.setBusinessScope(member.getBusinessScope());
		enterpriseUser.setCardName(member.getCardName());
		enterpriseUser.setCardNo(member.getCardNo());
		enterpriseUser.setLegalCertId(member.getLegalCertId());
		enterpriseUser.setLegalCertIdExpires(member.getLegalCertIdExpires());
		enterpriseUser.setLegalMp(member.getLegalMp());
		enterpriseUser.setLegalPerson(member.getLegalPerson());
		enterpriseUser.setMemberId(member.getMemberId());
		enterpriseUser.setName(member.getName());

		if (StringUtils.equals(env, "prod") || StringUtils.equals(env, "pre")) {
			enterpriseUser.setNotifyUrl(MessageFormatter.arrayFormat(AdapayNotifyConstants.corp_member_notify_url_prod, "adapay"));
		} else {
			enterpriseUser.setNotifyUrl(MessageFormatter.arrayFormat(AdapayNotifyConstants.corp_member_notify_url_test, "adapay"));
		}
		enterpriseUser.setProvCode(member.getProvCode());
		enterpriseUser.setSocialCreditCode(member.getSocialCreditCode());
		enterpriseUser.setSocialCreditCodeExpires(member.getSocialCreditCodeExpires());

		return corpMemberService.create(hfMerchantConfig.getMerKey(), hfMerchantConfig.getAppId(), enterpriseUser, enterpriseUser.getNotifyUrl());
	}
	
	private Map<String, Object> createMemberWithYeepay(YeepayMember member) throws IOException {

		HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(member.getTenantId(), member.getPayerName());
		if (hfMerchantConfig == null) {
			throw new BizException("机构[{}]易宝商户信息找不到", member.getTenantId());
		}
		
		MerchantInfo merchantInfo = new MerchantInfo();
		merchantInfo.setAddress(member.getAddress());
		merchantInfo.setBankAccountType(member.getBankAcctType());
		merchantInfo.setBankCardNo(member.getCardNo());
		merchantInfo.setBankCode(member.getBankCode());
		merchantInfo.setCityCode(member.getAreaCode());
		merchantInfo.setContactEmail(member.getContactEmail());
		merchantInfo.setContactLicenceNo(member.getContactLicenceNo());
		merchantInfo.setContactMobile(member.getContactMobile());
		merchantInfo.setContactName(member.getContactName());
		merchantInfo.setDistrictCode(member.getDistrictCode());
		merchantInfo.setLegalLicenceBackUrl(member.getLegalLicenceBackUrl());
		merchantInfo.setLegalLicenceFrontUrl(member.getLegalLicenceFrontUrl());
		merchantInfo.setLegalLicenceNo(member.getLegalCertId());
		merchantInfo.setLegalLicenceType(member.getLegalLicenceType());
		merchantInfo.setLegalName(member.getLegalPerson());
		merchantInfo.setLicenceNo(member.getSocialCreditCode());
		merchantInfo.setLicenceUrl(member.getLicenceUrl());
		merchantInfo.setMerchantType(member.getMerchantType());
		merchantInfo.setMerchantName(member.getName());
		merchantInfo.setMobile(member.getLegalMp());
		merchantInfo.setOpenAccountLicenceUrl(member.getOpenAccountLicenceUrl());
		merchantInfo.setProvinceCode(member.getProvCode());
		merchantInfo.setSettlementDirection("BANKCARD");

		String notifyURL = MessageFormatter.arrayFormat(AdapayNotifyConstants.corp_member_notify_url_test, "yeepay");
		if (StringUtils.equals(env, "prod") || StringUtils.equals(env, "pre")) {
			notifyURL = MessageFormatter.arrayFormat(AdapayNotifyConstants.corp_member_notify_url_prod, "yeepay");
		}
		
		return yeepayMerchantService.registerSaasMerchantV2(hfMerchantConfig, merchantInfo, notifyURL);
	}*/
}
