package com.ym.mec.biz.service.impl;

import com.google.common.collect.Lists;
import com.microsvc.toolkit.middleware.payment.common.api.PaymentServiceContext;
import com.microsvc.toolkit.middleware.payment.impl.AdapayPaymentServicePlugin;
import com.microsvc.toolkit.middleware.payment.impl.YeepayPaymentServicePlugin;
import com.microsvc.toolkit.middleware.payment.properties.PayConfigProperties;
import com.ym.mec.biz.dal.dao.HfMerchantConfigDao;
import com.ym.mec.biz.service.HfMerchantConfigService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.adapay.entity.HfMerchantConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;

@Slf4j
@Service
public class HfMerchantConfigServiceImpl extends BaseServiceImpl<Integer, HfMerchantConfig>  implements HfMerchantConfigService {
	
	@Resource
	private HfMerchantConfigDao hfMerchantConfigDao;
	@Resource
	private PayConfigProperties properties;

	@Override
	public BaseDAO<Integer, HfMerchantConfig> getDAO() {
		return hfMerchantConfigDao;
	}

	@PostConstruct
	public void init(){
		List<HfMerchantConfig> list = hfMerchantConfigDao.findAll(new HashMap<>());

		if(CollectionUtils.isEmpty(list)) {
			return;
		}

		List<String> pluginNames = Lists.newArrayList();
		for (HfMerchantConfig config : list) {
			if (StringUtils.isEmpty(config.getPayerName())) {
				log.error("数据错误 {}", config);
			} else if (StringUtils.equalsIgnoreCase(config.getPayerName(),"ADAPAY")) {
				PayConfigProperties.AdapayPayConfig adapayPayConfig = new PayConfigProperties.AdapayPayConfig();
				adapayPayConfig.setEnable(true);
				adapayPayConfig.setAppId(config.getAppId());
				adapayPayConfig.setPayType(config.getPayerName() + "_" + config.getTenantId());
				adapayPayConfig.setMerchantKey(config.getMerKey());
				adapayPayConfig.setMerchantId(config.getApiKey());
				adapayPayConfig.setPayNotifyUrl(properties.getPayNotifyUrl());
				adapayPayConfig.setRefundNotifyUrl(properties.getRefundNotifyUrl());
				adapayPayConfig.setKeyPublic(config.getRsaPublicKey());
				adapayPayConfig.setKeyPrivate(config.getRsaPrivateKey());
				adapayPayConfig.setWxAppId(config.getWxAppId());
				adapayPayConfig.setWxAppSecret(config.getWxAppSecret());
//				adapayPayConfig.setMiniAppId(config.getMiniAppId());
//				adapayPayConfig.setMiniAppSecret(config.getMiniAppSerret());
				adapayPayConfig.setSupportCreditCards(properties.getSupportCreditCards());
				try {
					AdapayPaymentServicePlugin plugin = new AdapayPaymentServicePlugin(adapayPayConfig.getPayType(), adapayPayConfig, false);
					PaymentServiceContext.addPlugin(plugin);
					pluginNames.add(plugin.venderName());
				} catch (Exception e) {
					log.error("初始[汇付]化支付插件失败", e);
				}
			} else if (StringUtils.equalsIgnoreCase(config.getPayerName(),"YEEPAY")){
				PayConfigProperties.YeepayPayConfig yeepayPayConfig = new PayConfigProperties.YeepayPayConfig();
				yeepayPayConfig.setEnable(true);
				yeepayPayConfig.setAppId(config.getAppId());
				yeepayPayConfig.setPayType(config.getPayerName() + "_" + config.getTenantId());
				yeepayPayConfig.setMerchantId(config.getPlatformPayeeMemberId());
				yeepayPayConfig.setPayNotifyUrl(properties.getPayNotifyUrl());
				yeepayPayConfig.setRefundNotifyUrl(properties.getRefundNotifyUrl());
				yeepayPayConfig.setKeyPublic(config.getRsaPublicKey());
				yeepayPayConfig.setKeyPrivate(config.getRsaPrivateKey());
				yeepayPayConfig.setWxAppId(config.getWxAppId());
				yeepayPayConfig.setWxAppSecret(config.getWxAppSecret());
				yeepayPayConfig.setSupportCreditCards(properties.getSupportCreditCards());
				yeepayPayConfig.setAlipayAppId(config.getAlipayAppId());
				yeepayPayConfig.setAlipayPrivateKey(config.getAlipayPrivateKey());
				yeepayPayConfig.setAlipayPublicKey(config.getAlipayPublicKey());
//				yeepayPayConfig.setMiniAppId(config.getMiniAppId());
//				yeepayPayConfig.setMiniAppSecret(config.getMiniAppSerret());
				try {
					YeepayPaymentServicePlugin plugin = new YeepayPaymentServicePlugin(yeepayPayConfig.getPayType(), yeepayPayConfig, false);
					PaymentServiceContext.addPlugin(plugin);

					pluginNames.add(plugin.venderName());
				} catch (Exception e) {
					log.error("初始化[易宝]支付插件失败", e);
				}
			}else {
				log.error("数据错误 {}", config);
			}
		}
		log.info("初始化支付插件完成, plugins={}", pluginNames);
	}

	/*@Override
	public void afterPropertiesSet() throws Exception {
		AdapayInit adapayInit = AdapayInit.getInstance(false, true);
		
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("payerName", PaymentChannelEnum.ADAPAY.getCode());
		
		adapayInit.initWithMerConfig(hfMerchantConfigDao.findAll(params));
	}*/

	@Override
	public HfMerchantConfig queryByTenantId(Integer tenantId, String payerName) {
		return hfMerchantConfigDao.queryByTenantId(tenantId, payerName);
	}
	
}