package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.MusicGroupPaymentCalenderMemberDao;
import com.ym.mec.biz.dal.dto.MusicGroupPaymentBaseCalender;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.CalenderBaseServiceEnum;
import com.ym.mec.biz.dal.enums.PeriodEnum;
import com.ym.mec.biz.service.MemberFeeSettingService;
import com.ym.mec.biz.service.MemberRankSettingService;
import com.ym.mec.biz.service.MusicGroupPaymentCalenderBaseService;
import com.ym.mec.biz.service.MusicGroupPaymentCalenderMemberService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.function.BiFunction;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PaymentCalenderStatusEnum.AUDITING;
import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PaymentCalenderStatusEnum.NO;

@Service
public class MusicGroupPaymentCalenderMemberServiceImpl extends BaseServiceImpl<Long, MusicGroupPaymentCalenderMember>
        implements MusicGroupPaymentCalenderBaseService<MusicGroupPaymentCalenderMember>, MusicGroupPaymentCalenderMemberService {

    @Resource
    private MusicGroupPaymentCalenderMemberDao musicGroupPaymentCalenderMemberDao;
    @Resource
    private MemberFeeSettingService memberFeeSettingService;
    @Resource
    private MemberRankSettingService memberRankSettingService;

    @Override
    public BaseDAO<Long, MusicGroupPaymentCalenderMember> getDAO() {
        return musicGroupPaymentCalenderMemberDao;
    }

    public MusicGroupPaymentCalenderMemberServiceImpl() {
        MusicGroupPaymentCalenderBaseService.calenderBaseServiceMap.put(CalenderBaseServiceEnum.MEMBER,this);
    }

    @Override
    public MusicGroupPaymentCalender.PaymentCalenderStatusEnum checkComponentAmount(MusicGroupPaymentBaseCalender baseCalender) {
        List<MusicGroupPaymentCalenderMember> calenderMemberList = baseCalender.getCalenderMemberList();
        if(CollectionUtils.isEmpty(calenderMemberList)){
            return NO;
        }
        for (MusicGroupPaymentCalenderMember calenderMember : calenderMemberList) {
            if(getAmount(baseCalender,calenderMember, MemberFeeSettingServiceImpl::getGroupPurchaseFee).
                    compareTo(calenderMember.getActualAmount()) != 0){
                return AUDITING;
            }
        }
        return NO;
    }

    @Override
    public BigDecimal getActualAmount(MusicGroupPaymentBaseCalender baseCalender) {
        List<MusicGroupPaymentCalenderMember> calenderMemberList = baseCalender.getCalenderMemberList();
        if(CollectionUtils.isNotEmpty(calenderMemberList)){
            return calenderMemberList.stream().map(MusicGroupPaymentCalenderBase::getActualAmount).reduce(BigDecimal.ZERO,BigDecimal::add);
        }
        return BigDecimal.ZERO;
    }

    @Override
    public BigDecimal getOriginalAmount(MusicGroupPaymentBaseCalender baseCalender) {
        return getAmount(baseCalender, MemberFeeSettingServiceImpl::getOriginalFee);
    }

    @Override
    public BigDecimal getCurrentAmount(MusicGroupPaymentBaseCalender calenderMember) {
        return getAmount(calenderMember, MemberFeeSettingServiceImpl::getGroupPurchaseFee);
    }

    @Override
    public <E> E initBean(E bean) {
        if(bean != null){
            MusicGroupPaymentCalenderMember calenderMember = (MusicGroupPaymentCalenderMember)bean;
            calenderMember.setName(memberRankSettingService.queryMemberName(calenderMember.getMemberRankSettingId()));
        }
        return bean;
    }

    private BigDecimal getAmount(MusicGroupPaymentBaseCalender baseCalender, BiFunction<PeriodEnum,MemberFeeSetting,BigDecimal> func){
        List<MusicGroupPaymentCalenderMember> calenderMemberList = baseCalender.getCalenderMemberList();
        if(CollectionUtils.isNotEmpty(calenderMemberList)){
            MusicGroup musicGroup = baseCalender.getMusicGroup();
            BigDecimal amount = BigDecimal.ZERO;
            for (MusicGroupPaymentCalenderMember calenderMember : calenderMemberList) {
                amount = amount.add(memberFeeSettingService.getAmount(musicGroup.getOrganId(), calenderMember.getMemberRankSettingId(),
                        calenderMember.getPeriod(), calenderMember.getNum(), func));
            }
            return amount;
        }
        return BigDecimal.ZERO;
    }

    private BigDecimal getAmount(MusicGroupPaymentBaseCalender baseCalender,MusicGroupPaymentCalenderMember calenderMember, BiFunction<PeriodEnum,MemberFeeSetting,BigDecimal> func){
        if(calenderMember != null){
            MusicGroup musicGroup = baseCalender.getMusicGroup();
            //会员价格是否变动
            return memberFeeSettingService.getAmount(musicGroup.getOrganId(),calenderMember.getMemberRankSettingId(),
                    calenderMember.getPeriod(),calenderMember.getNum(),func);
        }
        return BigDecimal.ZERO;
    }

    @Override
    public void deleteByCalenderId(Long calenderId) {
        musicGroupPaymentCalenderMemberDao.deleteByCalenderId(calenderId);
    }

    @Override
    public List<MusicGroupPaymentCalenderMember> findByCalenderId(Long calenderId) {
        List<MusicGroupPaymentCalenderMember> calenderMembers = musicGroupPaymentCalenderMemberDao.findByCalenderId(calenderId);
        if(CollectionUtils.isNotEmpty(calenderMembers)){
            List<Integer> memberIds = calenderMembers.stream().map(MusicGroupPaymentCalenderMember::getMemberRankSettingId).collect(Collectors.toList());
            List<MemberRankSetting> memberRankSettingList = memberRankSettingService.listByIds(memberIds);
            Map<Integer, MemberRankSetting> memberRankSettingMap = memberRankSettingList.stream().collect(Collectors.toMap(MemberRankSetting::getId, e -> e));
            calenderMembers.forEach(e -> {
                MemberRankSetting memberRankSetting = memberRankSettingMap.get(e.getMemberRankSettingId());
                if(memberRankSetting != null){
                    e.setIntro(memberRankSetting.getIntro());
                }
            });
        }
        return calenderMembers;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchInsert(List<MusicGroupPaymentCalenderMember> calenderMemberList) {
        musicGroupPaymentCalenderMemberDao.batchInsert(calenderMemberList);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchUpdate(List<MusicGroupPaymentCalenderMember> calenderMemberList) {
        musicGroupPaymentCalenderMemberDao.batchUpdate(calenderMemberList);
    }

    @Override
    public List<MusicGroupPaymentCalenderMember> findByIds(List<Long> calenderMemberIds) {
        List<MusicGroupPaymentCalenderMember> calenderMembers = musicGroupPaymentCalenderMemberDao.findByIds(calenderMemberIds);
        if(CollectionUtils.isNotEmpty(calenderMembers)){
            List<Integer> memberIds = calenderMembers.stream().map(MusicGroupPaymentCalenderMember::getMemberRankSettingId).collect(Collectors.toList());
            List<MemberRankSetting> memberRankSettingList = memberRankSettingService.listByIds(memberIds);
            Map<Integer, MemberRankSetting> memberRankSettingMap = memberRankSettingList.stream().collect(Collectors.toMap(MemberRankSetting::getId, e -> e));
            calenderMembers.forEach(e -> {
                MemberRankSetting memberRankSetting = memberRankSettingMap.get(e.getMemberRankSettingId());
                if(memberRankSetting != null){
                    e.setIntro(memberRankSetting.getIntro());
                }
            });
        }
        return calenderMembers;
    }
}
