package com.yonge.cooleshow.portal.controller.open;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.enums.ApiErrorCode;
import com.ym.mec.common.controller.BaseController;
import com.yonge.cooleshow.mall.common.api.CommonPage;
import com.yonge.cooleshow.mall.common.api.CommonResult;
import com.yonge.cooleshow.mall.common.courier.domain.CourierInfo;
import com.yonge.cooleshow.mall.common.courier.service.CourierService;
import com.yonge.cooleshow.mbg.model.PmsProduct;
import com.yonge.cooleshow.portal.domain.OmsOrderDetail;
import com.yonge.cooleshow.portal.domain.PmsPortalProductDetail;
import com.yonge.cooleshow.portal.dto.ProductSearch;
import com.yonge.cooleshow.portal.dto.ProductSearchConditionVo;
import com.yonge.cooleshow.portal.dto.StockOperateModel;
import com.yonge.cooleshow.portal.service.OmsOrderCourierService;
import com.yonge.cooleshow.portal.service.OmsPortalOrderService;
import com.yonge.cooleshow.portal.service.PmsPortalProductService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/open")
@Api(tags = "开放权限接口")
public class OpenShareController extends BaseController {
    private final static Logger log = LoggerFactory.getLogger(OpenShareController.class);

    @Autowired
    private PmsPortalProductService portalProductService;

    @Autowired
    private OmsPortalOrderService portalOrderService;

    @Autowired
    private OmsOrderCourierService orderCourierService;
    @Autowired
    private CourierService courierService;

    @ApiOperation(value = "同步库存")
    @PostMapping(value="/updateStock/{stockType}")
    @ResponseBody
    public CommonResult<String> updateStock(@RequestBody List<StockOperateModel> model, @PathVariable String stockType) {
        // 库存操作 stockType  add:添加   reduce:减少  lock:锁定  unlock:解锁
        portalProductService.reduceStock(model,stockType);
        return CommonResult.success("");
    }

    @ApiOperation(value = "综合搜索、筛选、排序")
    @RequestMapping(value = "/product/search", method = RequestMethod.POST)
    @ResponseBody
    public CommonResult<CommonPage<PmsProduct>> search(@RequestBody ProductSearch search) {
        List<PmsProduct> productList = portalProductService.search(search.getKeyword(),search.getBrandId(),search.getProductCategoryId(),
            search.getProductAttributeCategoryId(), search.getPageNum(), search.getPageSize(), search.getSort());
        return CommonResult.success(CommonPage.restPage(productList));
    }

    @ApiOperation("获取商品列表下拉条件")
    @RequestMapping(value = "/search/condition", method = RequestMethod.GET)
    @ResponseBody
    public CommonResult<ProductSearchConditionVo> searchCondition() {
        ProductSearchConditionVo list = portalProductService.searchCondition();
        return CommonResult.success(list);
    }

    @ApiOperation("获取前台商品详情")
    @RequestMapping(value = "/product/detail/{id}", method = RequestMethod.GET)
    @ResponseBody
    public CommonResult<PmsPortalProductDetail> detail(@PathVariable Long id,@RequestParam String productSn) {
        PmsPortalProductDetail productDetail = portalProductService.detail(id);
        return CommonResult.success(productDetail);
    }

    @PostMapping(value = "/product/queryNameMap", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public CommonResult<Map<Long,String>> queryNameMap(@RequestBody List<Long> goodsIds) {
        return CommonResult.success(portalProductService.queryNameMap(goodsIds));

    }


    /**
     * 商品库存查询
     * @param goodsSkuId 商品存储Id
     * @return String
     */
    @GetMapping(value = "/product/stocks/{id}", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public R<Integer> productStocks(@PathVariable(name = "id") Long goodsSkuId) {
        log.info("商品库存查询 skuId:{}",goodsSkuId);
        return R.ok(portalProductService.getStockBySkuId(goodsSkuId));

    }


    /**
     * 查询商品订单状态
     * @param orderNo 订单编号
     * @return String
     */
    @GetMapping(value = "/product/orderStatus/{id}", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public R<Boolean> productOrderStatus(@PathVariable(name = "id") String orderNo){
        log.info("查询商品订单状态:{}",orderNo);
        OmsOrderDetail detail = portalOrderService.detail(orderNo);

        return R.restResult(detail.getStatus().equals(1), ApiErrorCode.SUCCESS);
    }


    @ApiOperation(value = "快递信息回调")
    @PostMapping(value="/courierCallback", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    @ResponseBody
    public Object courierCallback( String param) {
        log.info("快递信息回调：{}",param);

        JSONObject jsonObject = JSON.parseObject(param);
        String data = jsonObject.getString("lastResult");
        CourierInfo courierInfos = courierService.getCourierInfos(data);

        // 保存快递信息
        orderCourierService.updateCourierInfo(courierInfos);
        Map<String,Object> result = new HashMap<>();
        result.put("result",true);
        result.put("returnCode","200");
        result.put("message","成功");

        return result;
    }


}
